package models

import (
	"encoding/json"
	"fmt"
	"io"
	"log"
	"net/http"
	"net/url"
	"os"
	"strings"
	"time"

	"forge.grandlyon.com/web-et-numerique/factory/llle_project/backoffice-server/internal/constants"
	"gorm.io/gorm"
)

type GrdfAccessToken struct {
	AccessToken string    `json:"access_token"`
	FetchedAt   time.Time `json:"fetched_at"`
}

// Fetches GRDF auth API for an access token and save it in DB
func FetchGRDFAuthAPI() {
	log.Println("| Calling GRDF auth API")

	dh := NewDataHandler()

	reqBody := url.Values{
		"scope":         {"/adict/v2"},
		"grant_type":    {"client_credentials"},
		"client_id":     {os.Getenv("GRDF_CLIENT_ID")},
		"client_secret": {os.Getenv("GRDF_CLIENT_SECRET")},
	}

	client := &http.Client{}
	req, err := http.NewRequest(http.MethodPost, "https://adict-connexion.grdf.fr/oauth2/aus5y2ta2uEHjCWIR417/v1/token", strings.NewReader(reqBody.Encode()))
	if err != nil {
		fmt.Println("Error creating request:", err)
		return
	}

	req.Header.Add("Content-Type", "application/x-www-form-urlencoded")

	resp, err := client.Do(req)
	if err != nil {
		fmt.Println("Error sending request:", err)
		return
	}
	defer resp.Body.Close()

	body, err := io.ReadAll(resp.Body)
	if err != nil {
		fmt.Println("Error reading response body:", err)
		return
	}

	var result map[string]interface{}
	if err := json.Unmarshal(body, &result); err != nil {
		fmt.Println("Error unmarshaling JSON:", err)
		return
	}

	accessToken, ok := result["access_token"].(string)
	if !ok || accessToken == "" {
		log.Println("Access token not found in response")
		return
	}

	dh.WriteAccessToken(accessToken)
}

// Writes a new access token to the database
func (dh *DataHandler) WriteAccessToken(token string) {
	result := dh.sqlClient.Session(&gorm.Session{AllowGlobalUpdate: true}).Model(&GrdfAccessToken{}).Updates(&GrdfAccessToken{
		AccessToken: token,
		FetchedAt:   time.Now(),
	})

	if result.Error != nil {
		log.Println("| Error updating GRDF access token")
		log.Println(result.Error)
	} else {
		log.Println("| New GRDF access token written")
	}
}

func (dh *DataHandler) GetGrdfAccessToken(w http.ResponseWriter, r *http.Request) {
	var token GrdfAccessToken
	err := dh.sqlClient.First(&token).Error

	if err != nil {
		w.WriteHeader(http.StatusInternalServerError)
		return
	}
	w.Header().Set(constants.ContentType, constants.Json)
	json.NewEncoder(w).Encode(token)
	log.Printf("| get grdfAccessToken | %v", r.RemoteAddr)
}
