package models

import (
	"encoding/json"
	"fmt"
	"net/http"
	"strconv"

	"forge.grandlyon.com/web-et-numerique/factory/llle_project/backoffice-server/internal/constants"
)

type MonthlyReport struct {
	Year        int    `json:"year"`
	Month       int    `json:"month"`
	Subject     string `json:"subject"`
	Info        string `json:"info"`
	Image       string `json:"image"`
	NewsTitle   string `json:"newsTitle"`
	NewsContent string `json:"newsContent"`
	Question    string `json:"question"`
	Link        string `json:"link"`
}

func (dh *DataHandler) GetMonthlyReport(w http.ResponseWriter, r *http.Request) {

	year, err := strconv.Atoi(r.URL.Query().Get("year"))
	if err != nil {
		http.Error(w, "year in query is not an integer", http.StatusBadRequest)
		return
	}
	month, err := strconv.Atoi(r.URL.Query().Get("month"))
	if err != nil {
		http.Error(w, "month in query is not an integer", http.StatusBadRequest)
		return
	}

	monthlyReport, err := dh.getMonthlyReport(year, month)
	if err != nil {
		w.WriteHeader(http.StatusNotFound)
		return
	}

	w.Header().Set(constants.ContentType, constants.Json)
	json.NewEncoder(w).Encode(monthlyReport)
}

func (dh *DataHandler) getMonthlyReport(year int, month int) (monthlyReport MonthlyReport, err error) {
	var monthlyInfo MonthlyInfo
	dh.sqlClient.Where("year = ? AND month = ?", year, month).First(&monthlyInfo)

	var mailSubject MailSubject
	err = dh.sqlClient.Where("year = ? AND month = ?", year, month).First(&mailSubject).Error
	if err != nil || mailSubject.Subject == "" {
		// Build default object
		mailSubject.Subject = fmt.Sprintf("[Ecolyo] Votre bilan %s", getDateWithPrep(month-1, year))
	}

	var monthlyNews MonthlyNews
	dh.sqlClient.Where("year = ? AND month = ?", year, month).First(&monthlyNews)

	var poll Poll
	dh.sqlClient.Where("year = ? AND month = ?", year, month).First(&poll)

	monthlyReport = MonthlyReport{
		Year:        year,
		Month:       month,
		Subject:     mailSubject.Subject,
		Info:        monthlyInfo.Info,
		Image:       monthlyInfo.Image,
		NewsTitle:   monthlyNews.Title,
		NewsContent: monthlyNews.Content,
		Question:    poll.Question,
		Link:        poll.Link,
	}

	return monthlyReport, nil
}

func getDateWithPrep(month int, year int) string {
	frenchMonths := []string{
		"de janvier",
		"de février",
		"de mars",
		"d'avril",
		"de mai",
		"de juin",
		"de juillet",
		"d'août",
		"de septembre",
		"d'octobre",
		"de novembre",
		"de décembre",
	}

	var prevMonth = month - 1
	var prevYear = year
	// If month is January return December of the previous year
	if month == 0 {
		prevMonth = 11
		prevYear = year - 1
	}

	return fmt.Sprintf("%s %d", frenchMonths[prevMonth], prevYear)
}
